//**************************************************************************/
// Copyright (c) 1998-2007 Autodesk, Inc.
// All rights reserved.
// 
// These coded instructions, statements, and computer programs contain
// unpublished proprietary information written by Autodesk, Inc., and are
// protected by Federal copyright law. They may not be disclosed to third
// parties or copied or duplicated in any form, in whole or in part, without
// the prior written consent of Autodesk, Inc.
//**************************************************************************/
// DESCRIPTION: Includes for Plugins
// AUTHOR: Felix Joleanes
//***************************************************************************/
#include "AnimatablePivot.h"
#include "ControlContainer.h"

class AnimPivotTarget : public Control {
	public:
		void DeleteThis() { delete this; }	

		//references
		ControlContainer *pivotCtrl;
		ControlContainer *TMCtrl;	
		IParamBlock2	*pblock;
		IParamBlock2	*stepPblock;

		//offset controller
		ControlContainer *offsetCtrl;

		IObjParam *ip;
		HWND hwnd;
		
		//Keyframe interfaces
		IKeyControl *pivotKeys;
		IKeyControl *offsetKeys;

		//Animatable Methods
		Class_ID ClassID() {return AnimPivotTarget_CLASS_ID;}		
		SClass_ID SuperClassID() { return CTRL_MATRIX3_CLASS_ID; }
		void GetClassName(TSTR& s) {s = GetString(IDS_TARGET_CLASS_NAME);}
		int NumSubs() { return NSUBS;}
		Animatable* SubAnim(int i);
		TSTR SubAnimName(int i);
		int SubNumToRefNum(int subNum) { return (subNum);}
		void MouseCycleStarted(TimeValue t);
		void MouseCycleCompleted(TimeValue t);

		//to display controller params on the motion panel 
		void BeginEditParams( IObjParam *ip, ULONG flags,Animatable *prev ); 
		// to undisplay 
		void EndEditParams( IObjParam *ip, ULONG flags,Animatable *next ); 
		// return number of ParamBlocks in this instance 
		int NumParamBlocks() {return NPBLOCKS;} // return i'th ParamBlock 
		IParamBlock2* GetParamBlock(int i) {if (i==0) return pblock; else return NULL;} 
		// return id'd ParamBlock 
		IParamBlock2* GetParamBlockByID(BlockID id) {
			if (pblock->ID()==id)
				return pblock;
			return NULL;
		}

		//Reference Methods
		int NumRefs() {return NTARGETREFS;}
		RefTargetHandle GetReference(int i);
		void SetReference(int i, RefTargetHandle rtarg);

#if IS_SUPMAX2015
		RefResult NotifyRefChanged(const Interval& changeInt, RefTargetHandle hTarget, 
			PartID& partID,  RefMessage message, BOOL propagate);
#else
		RefResult NotifyRefChanged(Interval changeInt, RefTargetHandle hTarget, 
			PartID& partID,  RefMessage message);
#endif

		IOResult Load(ILoad *iload) {return IO_OK;}
		IOResult Save(ISave *isave) {return IO_OK;}
		void RescaleWorldUnits(float f) {}


		//Control Methods
		void Copy(Control *from);
		RefTargetHandle Clone( RemapDir &remap );
		int IsKeyable(){return 1;}
		BOOL IsLeaf(){return FALSE;}
		void GetValue(TimeValue t, void *val, Interval &valid, GetSetMethod method = CTRL_ABSOLUTE );
		void SetValue(TimeValue t, void *val, int commit=1, GetSetMethod method= CTRL_ABSOLUTE);
		void CommitValue(TimeValue t);
		void RestoreValue(TimeValue t);
		Control* GetPositionController();
		Control* GetRotationController();
		Control* GetScaleController();
		BOOL SetPositionController(Control *c);
		BOOL SetRotationController(Control *c);
		BOOL SetScaleController(Control *c);
		BOOL ChangeParents  (TimeValue  t,const Matrix3 &oldP, const Matrix3 &newP, const Matrix3 &tm);

		//StepOffset Methods
		//these methods are used to establish an extra position for the pivot in step intervals (in step keyframes)
		//to avoid gaps in the node position when there is a step keyframe with a current TMCtrl value
		//diferent from the previous keyframe.
		//if a new keyframe is created in the pivotCtrl it's value needs to be changed to match this extra interpolation
		void SOGetValue(TimeValue t, void *val, GetSetMethod method = CTRL_ABSOLUTE);
		Point3 SOGetStepBezier(float t,Point3 val);

		//Constructor/Destructor
		AnimPivotTarget(BOOL loading=FALSE);
		AnimPivotTarget(ControlContainer *newPivotCtrl,ControlContainer *newTMControl,IParamBlock2 *theStepPblock,ControlContainer *newOffsetCtrl);
		~AnimPivotTarget() {}
};

class AnimPivotTargetClassDesc : public ClassDesc2 
{
public:
	virtual int IsPublic() 							{ return FALSE; }
	virtual void* Create(BOOL loading) 		{return new AnimPivotTarget(loading); }
	void DeleteThis() { delete this; }
	virtual const TCHAR *	ClassName() 			{ return GetString(IDS_TARGET_CLASS_NAME); }
	virtual SClass_ID SuperClassID() 				{ return CTRL_MATRIX3_CLASS_ID; }
	virtual Class_ID ClassID() 						{ return AnimPivotTarget_CLASS_ID; }
	virtual const TCHAR* Category() 				{ return _T(""); }

	virtual const TCHAR* InternalName() 			{ return _T("AnimPivotTarget"); }	// returns fixed parsable name (scripter-visible name)
	virtual HINSTANCE HInstance() 					{ return hInstance; }					// returns owning module handle
	

};

class AnimPivotTargetDlgProc: public ParamMap2UserDlgProc
{
public:

	AnimPivotTarget *animpivottarget;
	AnimPivotTargetDlgProc(AnimPivotTarget *Animpivottarget){animpivottarget=Animpivottarget;}
	virtual INT_PTR  DlgProc (TimeValue t, IParamMap2 *map, HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
	virtual void  DeleteThis (){delete this;}
};


class AnimPivotTargetClassAccessor : public PBAccessor
{ 
	public:
		void Set(PB2Value& v,ReferenceMaker* owner,ParamID id,int tabIndex,TimeValue  t);
};